#! /usr/bin/env perl

# Generate a number of .fig files containing a hatched region.
# One file per hatch color.  Region size: 29.7 x 1 cm in size
# (just enough to be clipped into a thin band at most 29.7 cm long).
# Hatch lines are inclined at 45 deg, from bottom left to top right.
# They "stick out" by thickness/sqrt(2) and should be clipped to the
# proper boundary.

use 5.006_001;
use strict;
use warnings;
use POSIX qw(ceil);

our $prefix = "logo-hatches-";
our $suffix = ".fig";
our %colors = (
	"rouge" => "#cd0039",  # from CMYK de004b, measured cd0039, was d93244
	"rose" => "#e08f89",   # from CMYK f29cbd, measured e08f89
	"orange" => "#ff6600", # from CMYK ff6b24, measured ff6600, was f86118
	"blue" => "#660066",   # from CMYK 6d02f0, measured 660066, was 660066
	"green" => "#669933"   # from CMYK 72dc10, measured 669933, was 669933
	);
our $width = 13365;	# Fig units, 45 per mm at this resolution:
our $height = 450;	# 1200 per "inch", but with 1 cm == 3/8 in!!
our $hatch_interval = 24;

sub fig_prefix();	# Return Fig format header.
sub hatches($$$$);	# Return Fig hatches of given wd, ht, interval, color.

while (my ($color_name, $color_hex) = each(%colors)) {
	my $file_name = "$prefix$color_name$suffix";
	open(my $fh, ">", $file_name)
		or die("Could not write to $file_name");
	print $fh fig_prefix()
		. hatches($width, $height, $hatch_interval, $color_hex);
	close($fh);
}
exit(0);

#
# Subroutines.
#
sub fig_prefix() {
	return <<"EOF";
#FIG 3.2
Portrait
Center
Metric
A4      
100.00
Single
-2
1200 2
EOF
}

sub hatches($$$$) {
	my ($width, $height, $interval, $color_hex) = (@_);
	my $text = "0 32 $color_hex\n"; # User-defined color object.
	my $line_text = "2 1 0 1 32 7 50 -1 -1 0.000 0 0 -1 0 0 2";
	my $line_increment = ceil($interval * sqrt(2));
	my $line_margin = ceil(15 / sqrt(2));
	
	# Top-left half-square.
	for (my $y = $height - $line_increment; $y >= 0;
	     $y -= $line_increment) {
		my $x1 =  0 - $line_margin;
		my $y1 = $y + $line_margin;
		my $x2 = $y + $line_margin;
		my $y2 =  0 - $line_margin;
		$text .= "$line_text\n\t $x1 $y1 $x2 $y2\n";
	}
	# Remainder.
	for (my $x = 0; $x <= $width; $x += $line_increment) {
		my $x1 = $x;
		my $y1 = $height;
		my $x2 = $x + $height;
		my $y2 = 0;
		if ($x2 > $width) {
			$x2 = $width;
			$y2 = $y1 - ($x2 - $x1);
		}
		$x1 -= $line_margin;
		$y1 += $line_margin;
		$x2 += $line_margin;
		$y2 -= $line_margin;
		$text .= "$line_text\n\t $x1 $y1 $x2 $y2\n";
	}
	return $text;
}
